"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular/core");
const sortablejs_module_1 = require("./sortablejs.module");
// Sortable
let Sortable = require('sortablejs');
// original library calls the events in unnatural order
// first the item is added, then removed from the previous array
// this is a temporary event to work this around
// as long as only one sortable takes place at a certain time
// this is enough to have a single `global` event
let onremove;
class SortablejsDirective {
    constructor(element, zone) {
        this.element = element;
        this.zone = zone;
        this.runInsideAngular = false;
    }
    ngOnInit() {
        if (this.runInsideAngular) {
            this._sortable = new Sortable(this.element.nativeElement, this.options);
        }
        else {
            this.zone.runOutsideAngular(() => {
                this._sortable = new Sortable(this.element.nativeElement, this.options);
            });
        }
    }
    ngOnChanges(changes) {
        const optionsChange = changes['inputOptions'];
        if (optionsChange && !optionsChange.isFirstChange()) {
            const previousOptions = optionsChange.previousValue;
            const currentOptions = optionsChange.currentValue;
            Object.keys(currentOptions).forEach(optionName => {
                if (currentOptions[optionName] !== previousOptions[optionName]) {
                    // use low-level option setter
                    this._sortable.option(optionName, currentOptions[optionName]);
                }
            });
        }
    }
    ngOnDestroy() {
        this._sortable.destroy();
    }
    get options() {
        return Object.assign({}, sortablejs_module_1.SortablejsModule._globalOptions, this.inputOptions, this.overridenOptions);
    }
    proxyEvent(eventName, event) {
        if (this.inputOptions && this.inputOptions[eventName]) {
            this.inputOptions[eventName](event);
        }
    }
    // returns whether the items are currently set
    get bindingEnabled() {
        return !!this.items;
    }
    // we need this to identify that the input is a FormArray
    // we don't want to have a dependency on @angular/forms just for that
    get isItemsFormArray() {
        // just checking for random FormArray methods not available on a standard array
        return !!this.items.at && !!this.items.insert && !!this.items.reset;
    }
    get overridenOptions() {
        // always intercept standard events but act only in case items are set (bindingEnabled)
        // allows to forget about tracking this.items changes
        return {
            onAdd: (event) => {
                if (this.bindingEnabled) {
                    onremove = (item) => {
                        if (this.isItemsFormArray) {
                            this.items.insert(event.newIndex, item);
                        }
                        else {
                            this.items.splice(event.newIndex, 0, item);
                        }
                    };
                }
                this.proxyEvent('onAdd', event);
            },
            onRemove: (event) => {
                if (this.bindingEnabled) {
                    let item;
                    if (this.isItemsFormArray) {
                        item = this.items.at(event.oldIndex);
                        this.items.removeAt(event.oldIndex);
                    }
                    else {
                        item = this.items.splice(event.oldIndex, 1)[0];
                    }
                    onremove(item);
                    onremove = null;
                }
                this.proxyEvent('onRemove', event);
            },
            onUpdate: (event) => {
                if (this.bindingEnabled) {
                    if (this.isItemsFormArray) {
                        let relocated = this.items.at(event.oldIndex);
                        this.items.removeAt(event.oldIndex);
                        this.items.insert(event.newIndex, relocated);
                    }
                    else {
                        this.items.splice(event.newIndex, 0, this.items.splice(event.oldIndex, 1)[0]);
                    }
                }
                this.proxyEvent('onUpdate', event);
            }
        };
    }
}
SortablejsDirective.decorators = [
    { type: core_1.Directive, args: [{
                selector: '[sortablejs]'
            },] },
];
/** @nocollapse */
SortablejsDirective.ctorParameters = () => [
    { type: core_1.ElementRef, },
    { type: core_1.NgZone, },
];
SortablejsDirective.propDecorators = {
    'items': [{ type: core_1.Input, args: ['sortablejs',] },],
    'inputOptions': [{ type: core_1.Input, args: ['sortablejsOptions',] },],
    'runInsideAngular': [{ type: core_1.Input },],
};
exports.SortablejsDirective = SortablejsDirective;
//# sourceMappingURL=sortablejs.directive.js.map